@@ -7,7 +7,7 @@ from sirius_sdk.agent.pairwise import Pairwise
 from sirius_sdk.agent.wallet.abstract.crypto import AbstractCrypto
 from sirius_sdk.agent.pairwise import AbstractPairwiseList
 from sirius_sdk.agent.microledgers import Transaction, Microledger, MicroledgerList
-from sirius_sdk.agent.sm import AbstractStateMachine, StateMachineTerminatedWithError
+from sirius_sdk.agent.sm import AbstractStateMachine, StateMachineTerminatedWithError, StateMachineAborted
 from sirius_sdk.agent.aries_rfc.feature_0015_acks import Ack, Status
 from sirius_sdk.agent.consensus.simple.messages import *
 
@@ -21,17 +21,22 @@ RESPONSE_PROCESSING_ERROR = 'response_processing_error'
 class MicroLedgerSimpleConsensus(AbstractStateMachine):
 
     def __init__(
-            self, crypto: AbstractCrypto, me: Pairwise.Me,
-            pairwise_list: AbstractPairwiseList, microledgers: MicroledgerList, *args, **kwargs
+            self, me: Pairwise.Me,
+            crypto: AbstractCrypto = None, pairwise_list: AbstractPairwiseList = None,
+            microledgers: MicroledgerList = None, *args, **kwargs
     ):
         self.__me = me
         self.__problem_report = None
         self.__microledgers = microledgers
+        self.__microledgers_internal = microledgers is None
         self.__crypto = crypto
+        self.__crypto_internal = crypto is None
         self.__pairwise_list = pairwise_list
+        self.__pairwise_list_internal = pairwise_list is None
         self.__transport = None
         self.__thread_id = None
         self.__cached_p2p = {}
+        self.__neighbours = []
         super().__init__(*args, **kwargs)
 
     @property
@@ -64,6 +69,7 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
         participants = list(set(participants + [self.me.did]))
         await self._setup('simple-consensus-' + uuid.uuid4().hex, self.time_to_live)
         try:
+            self.__neighbours = [did for did in participants if did != self.me.did]
             await self.log(progress=0, message=f'Create ledger [{ledger_name}]')
             ledger, genesis = await self.microledgers.create(ledger_name, genesis)
             await self.log(message=f'Ledger creation terminated successfully')
@@ -96,6 +102,7 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
         time_to_live = propose.timeout_sec or self.time_to_live
         await self._setup(propose.thread_id, time_to_live)
         try:
+            self.__neighbours = [did for did in propose.participants if did != self.me.did]
             ledger_name = propose.ledger.get('name', None)
             try:
                 if not ledger_name:
@@ -141,6 +148,7 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
     ) -> (bool, Optional[List[Transaction]]):
         await self._setup('simple-consensus-txns-' + uuid.uuid4().hex, self.time_to_live)
         try:
+            self.__neighbours = [did for did in participants if did != self.me.did]
             try:
                 await self.log(progress=0, message=f'Start committing {len(transactions)} transactions')
                 txns = await self._commit_internal(ledger, transactions, participants)
@@ -169,6 +177,7 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
         await self._setup(propose.thread_id, time_to_live)
         try:
             ledger = None
+            self.__neighbours = [did for did in propose.participants if did != self.me.did]
             try:
                 await self.log(progress=0, message=f'Start acception {len(propose.transactions)} transactions')
                 ledger = await self._load_ledger(actor, propose)
@@ -194,6 +203,15 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
         finally:
             await self._clean()
 
+    async def abort(self):
+        await super().abort()
+        if self.__transport and self.__transport.is_started:
+            await self._terminate_with_problem_report(
+                problem_code=REQUEST_PROCESSING_ERROR,
+                explain='Operation is aborted by owner',
+                their_did=self.__neighbours
+            )
+
     async def _switch(
             self, their_did: str, req: Union[SimpleConsensusMessage, SimpleConsensusProblemReport, Ack]
     ) -> (bool, Union[SimpleConsensusMessage, SimpleConsensusProblemReport]):
@@ -201,6 +219,9 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
             pw = await self.get_p2p(their_did)
             self.__transport.pairwise = pw
             ok, resp = await self.__transport.switch(req)
+            if self.is_aborted:
+                await self.log(progress=100, message='Aborted')
+                raise StateMachineAborted
             if isinstance(resp, SimpleConsensusMessage) or isinstance(resp, SimpleConsensusProblemReport) or isinstance(resp, Ack):
                 return ok, resp
             else:
@@ -255,11 +276,23 @@ class MicroLedgerSimpleConsensus(AbstractStateMachine):
         self.__thread_id = thread_id
         self.__transport = await self.transports.spawn(thread_id)
         await self.__transport.start(time_to_live=time_to_live)
+        if self.__microledgers_internal:
+            self.__microledgers = self.__transport.microledgers
+        if self.__crypto_internal:
+            self.__crypto = self.__transport.wallet.crypto
+        if self.__pairwise_list_internal:
+            self.__pairwise_list = self.__transport.pairwise_list
 
     async def _clean(self):
         if self.__transport:
             await self.__transport.stop()
         self.__transport = None
+        if self.__microledgers_internal:
+            self.__microledgers = None
+        if self.__crypto_internal:
+            self.__crypto = None
+        if self.__pairwise_list_internal:
+            self.__pairwise_list = None
 
     async def get_p2p(self, their_did: str, raise_exception: bool = True) -> Pairwise:
         if their_did not in self.__cached_p2p.keys():
